FUNCTION GKVsd::Execute, arg, arg1, arg2
;
; Purpose:
;
;		Applies any legal IDL function to argument of 'self'
;		and returns the result as a new GKV object.
;
; Arguments:
;
;		Accepts a string argument containing the name of an IDL function.
;
; Output:
;
;		Returns a GKV object of same dimensionality as 'self' whose
;		values are 'arg'(self.values)
;
; Written by W.M. Nevins
;	8/21/00
;
; Check argument
;
IF(TypeOf(arg) NE 7) THEN BEGIN
	MESSAGE, "Requires STRING argument containing the name of an IDL function", /INFORMATIONAL
	RETURN, 0
ENDIF
result = self -> MakeCopy(/NoErrorBars)
SelfValues = *result.values
nArgs = N_PARAMS()
CASE nArgs OF
	1:	BEGIN
		commandStr = 'ResultValues = ' + arg + '(SelfValues)'
		ok = EXECUTE(commandStr)
		END
	2:	BEGIN
		commandStr = 'ResultValues = ' + arg + '(SelfValues, arg1)'
		ok = EXECUTE(commandStr)
		END
	3:	BEGIN
		commandStr = 'ResultValues = ' + arg + '(SelfValues, arg1, arg2)'
		ok = EXECUTE(commandStr)
		END
	
ENDCASE
IF(NOT ok) THEN BEGIN
	MESSAGE, "Error executing " + arg, /INFORMATIONAL
	result -> trash
	RETURN, 0
ENDIF
PTR_FREE, result.values
result.values = PTR_NEW(ResultValues)
result.vrange = [GKVsd_MIN(ResultValues, MAX=vmax), vmax]
result.title = arg + '(' + self.title + ')'
result.mnemonic = arg + '_' + self.mnemonic
result.units = ''
RETURN, result
END ; ****** Execute ****** ;

