FUNCTION GKVs1D::TimesExp, arg, Gamma=gammmaIn, tAxis=tAxis
;
; Purpose:
;
;	Multiplies 'self' by exp(gamma*t)
;
; Argument:
;	
;		Gamma can be specified as an argument to this function; OR
;
; Keywords:
;
;	Gamma	Gamma can be specified by using the keyword 'gamma'.  
;		If BOTH an argument AND the keyword 'gamma' are supplied,
;		then this routine will use the value of gamma supplied by
;		the keyword.
;
;	tAxis	The number of the axis corresponding to 'time'.  Normally,
;		it is not necessary to set  this keyword, as the routine
;		will find the tAxis by searcing for its mnemonic, 't'.  
;		This keyword is provided only to over ride this shearch.
;
; Written by W.M. Nevins
;	11/29/01
;
; 
nArgs = N_PARAMS()
IF(nArgs) THEN gamma = arg
IF(KEYWORD_SET(gammmaIn)) THEN gamma=gammaIn
nDims = self -> NUMDIMS()

IF(N_ELEMENTS(tAxis) NE 1) THEN tAxis = self -> AxisNumber('t')

IF((tAxis EQ 0) OR (tAxis GT nDims)) THEN BEGIN
	MESSAGE, "Couldn't find time axis, or bad axis specifier", /INFORMATIONAL
	RETURN, 0
ENDIF

result = self -> MAKECOPY(/NoErrorBars)
result -> Restrict

CASE tAxis OF
	1	:	tGrid = result.grid1
	2	:	tGrid = result.grid2
	3	:	tGrid = result.grid3
	ELSE	:	BEGIN
				MESSAGE, "tAxis GT 3 (How can this happen???)", /INFORMATIONAL
				RETURN, 0
			END
ENDCASE

t = *(tGrid.values)

expT = EXP(gamma*t)

info = SIZE(*result.values)

CASE nDims OF 
	1:			expTarr = expT
	2:	BEGIN
		CASE tAxis OF
			1:	expTarr = expT#REPLICATE(1.,info[2])
			2:	expTarr = REPLICATE(1.,info[1])#expT
		ENDCASE
		END
	3:	BEGIN
			expTarr = FLTARR(info[1], info[2], info[3])
			CASE tAxis OF
				1:	FOR k=0,info[3]-1 DO expTarr[*,*,k] = expT#REPLICATE(1.,info[2])
				2:	FOR k=0,info[3]-1 DO expTarr[*,*,k] = REPLICATE(1.,info[1])#expT
				3:	FOR i=0,info[1]-1 DO expTarr[i,*,*] = REPLICATE(1.,info[2])#expT
			ENDCASE
		END
ENDCASE

resultValues = expTarr*(*result.values)

PTR_FREE, result.values

result.values = PTR_NEW(resultValues)

RETURN, result

END